import type { DCloneHTMLElement, DId, DSize } from '../../utils/types';
import type { DFormControl } from '../form';
import type { DSelectItem } from '../select';
import type { AbstractTreeNode, TreeOrigin } from '../tree/abstract-node';
import React from 'react';
import { TREE_NODE_KEY } from '../tree/utils';
export interface DCascaderRef {
    updatePosition: () => void;
}
export type DSearchItem<V extends DId, T extends TreeOrigin> = DSelectItem<V> & {
    [TREE_NODE_KEY]: AbstractTreeNode<V, T>;
};
export interface DCascaderItem<V extends DId> {
    label: string;
    value: V;
    loading?: boolean;
    disabled?: boolean;
    children?: DCascaderItem<V>[];
}
export interface DCascaderProps<V extends DId, T extends DCascaderItem<V>> extends React.HTMLAttributes<HTMLDivElement> {
    dRef?: {
        input?: React.ForwardedRef<HTMLInputElement>;
    };
    dFormControl?: DFormControl;
    dList: T[];
    dModel?: V | null | V[];
    dVisible?: boolean;
    dPlaceholder?: string;
    dSize?: DSize;
    dLoading?: boolean;
    dSearchable?: boolean;
    dSearchValue?: string;
    dClearable?: boolean;
    dDisabled?: boolean;
    dMultiple?: boolean;
    dOnlyLeafSelectable?: boolean;
    dCustomItem?: (item: T) => React.ReactNode;
    dCustomSelected?: (select: T) => string;
    dCustomSearch?: {
        filter?: (value: string, item: T) => boolean;
        sort?: (a: T, b: T) => number;
    };
    dPopupClassName?: string;
    dInputRender?: DCloneHTMLElement<React.InputHTMLAttributes<HTMLInputElement>>;
    onModelChange?: (value: any, item: any) => void;
    onVisibleChange?: (visible: boolean) => void;
    onSearchValueChange?: (value: string) => void;
    onClear?: () => void;
    onFirstFocus?: (value: T['value'], item: T) => void;
    afterVisibleChange?: (visible: boolean) => void;
}
declare function Cascader<V extends DId, T extends DCascaderItem<V>>(props: DCascaderProps<V, T>, ref: React.ForwardedRef<DCascaderRef>): JSX.Element | null;
export declare const DCascader: <V extends DId, T extends DCascaderItem<V>>(props: DCascaderProps<V, T> & React.RefAttributes<DCascaderRef>) => ReturnType<typeof Cascader>;
export {};
