import type { FormControlStatus } from './abstract-control';
import type { AsyncValidatorFn, ValidatorFn } from './validators';
import { AbstractControl } from './abstract-control';
type Mutable<T> = {
    -readonly [P in keyof T]: T[P];
};
type GetFormControlPropertyFromArray<T, A> = Mutable<A> extends [infer K, ...infer R] ? K extends keyof T ? GetFormControlPropertyFromArray<T[K], R> : any : T;
type GetFormControlProperty<T, S> = S extends `${infer K}.${infer R}` ? K extends keyof T ? GetFormControlProperty<T[K], R> : any : S extends keyof T ? T[S] : any;
type GetFormGroupT<C extends FormGroup<any>> = C extends FormGroup<infer T> ? T : unknown;
type GetFormGroupValue<T extends {
    [K in keyof T]: AbstractControl;
}> = {
    [K in keyof T]?: T[K] extends FormGroup ? GetFormGroupValue<GetFormGroupT<T[K]>> : T[K]['value'];
};
export declare class FormGroup<T extends {
    [K in keyof T]: AbstractControl;
} = any> extends AbstractControl<{
    [K in keyof T]: T[K]['value'];
}> {
    controls: T;
    protected _value: {
        [K in keyof T]: T[K]['value'];
    };
    protected _status: FormControlStatus;
    constructor(controls: T, validators?: ValidatorFn | ValidatorFn[] | null, asyncValidator?: AsyncValidatorFn | AsyncValidatorFn[] | null);
    get<S extends string>(path: S): AbstractControl<GetFormControlProperty<{
        [K in keyof T]: T[K]['value'];
    }, S>>;
    get<S extends ArrayLike<string>>(path: S): AbstractControl<GetFormControlPropertyFromArray<{
        [K in keyof T]: T[K]['value'];
    }, S>>;
    getError(errorCode: string, path?: string[] | string): any;
    hasError(errorCode: string, path?: string[] | string): boolean;
    addControl<K extends keyof T>(name: K, control: T[K]): void;
    addControl(name: string, control: AbstractControl): void;
    removeControl<K extends keyof T>(name: K): void;
    removeControl(name: string): void;
    setControl<K extends keyof T>(name: K, control: T[K]): void;
    setControl(name: string, control: AbstractControl): void;
    contains<K extends keyof T>(controlName: K): boolean;
    contains(controlName: string): boolean;
    setValue(value: {
        [K in keyof T]: T[K]['value'];
    } & {
        [K: string]: any;
    }, onlySelf?: boolean): void;
    patchValue(value: GetFormGroupValue<T> & {
        [K: string]: any;
    }, onlySelf?: boolean): void;
    reset(value?: GetFormGroupValue<T> & {
        [K: string]: any;
    }, onlySelf?: boolean): void;
    protected _updateValue(): void;
    protected _forEachChild(cb: (v: any, k: string) => void): void;
    protected _anyControls(condition: (c: AbstractControl) => boolean): boolean;
    protected _allControlsDisabled(): boolean;
    private _setUpControls;
    private _reduceValue;
    private _reduceChildren;
    private _checkAllValuesPresent;
    private _throwIfControlMissing;
}
export {};
